﻿using UnityEngine;
using System.Collections;

public enum FormationType
{
    INVALID_FORMATION = 0,
    BASIC_WEDGE
}

public abstract class Formation {

    protected Vector3[] mUnitPositions;
    protected Vector3 mExpectedCenterOfMass;

    private FormationType mType;
    private int mMaxUnits;

    public FormationType Type
    {
        get { return mType; }
    }
    public int MaxUnits
    {
        get { return mMaxUnits; }
    }
    public Vector3 ExpectedCenterOfMass
    {
        get { return mExpectedCenterOfMass; }
    }
    public float CenterOfMassOffset
    {
        get { return mExpectedCenterOfMass.magnitude; }
    }
    public float CenterOfMassOffsetSqr
    {
        get { return mExpectedCenterOfMass.sqrMagnitude; }
    }

	public Formation(int maxUnits, FormationType type = FormationType.INVALID_FORMATION)
    {
        mType = type;
        mMaxUnits = maxUnits;
        mUnitPositions = new Vector3[mMaxUnits];

        AssignPositions();
    }

    /// <summary>
    /// Returns the position for a specific unit
    /// </summary>
    /// <param name="unitIndex"></param>
    /// <returns></returns>
    virtual public Vector3 GetUnitPosition(int unitIndex)
    {
        Vector3 unitPos = Vector3.zero;

        if (unitIndex > mUnitPositions.Length - 1)
        {
            Debug.LogError("ERROR: Index of unit position requested is too large!");
            Debug.Break();
            return unitPos;
        }

        return mUnitPositions[unitIndex];
    }

    /// <summary>
    /// Assign the positions for each unit relative to virtual leader root and calculates center of mass offset
    /// </summary>
    abstract protected void AssignPositions();
}
