﻿using UnityEngine;
using System.Collections;

public class SquadMemberBehavior : MonoBehaviour {

    public VirtualLeader leaderObject;
    public float maxDrift = 1.0f;
    public float repathDistance = 1.0f;
    public float maxSpeedModifier = 2.0f;

    private NavMeshAgent mAgent;
    private VirtualLeader mMyLeader;
    private Vector3 mTargetPos;
    private Vector3 mFormationPos;
    private float mOriginalSpeed;
    private float mSpeedModifier;

	// Use this for initialization
	void Start ()
	{
        if (leaderObject == null)
        {
            Debug.LogError("ERROR: Leader not found for" + gameObject.name + "!");
            Debug.Break();
        }

        mMyLeader = leaderObject.GetComponent<VirtualLeader>();
        mAgent = GetComponent<NavMeshAgent>();
        mOriginalSpeed = mAgent.speed;

        StartCoroutine(CheckSquadTargetPosition());
	}

	// Update is called once per frame
	void Update () 
	{
        CheckSpeed();
	}

    /// <summary>
    /// Tells squad member to move to the specified position
    /// </summary>
    /// <param name="targetPos"></param>
    public void MoveTo(Vector3 targetPos)
    {
        mTargetPos = targetPos;
        mAgent.SetDestination(mTargetPos);
    }

    /// <summary>
    /// Changes the units speed based on how close they are to their position in the formation
    /// </summary>
    private void CheckSpeed()
    {
        float distToFormationPosZ = transform.InverseTransformPoint(mFormationPos).z;
        float distToFormationPosX = transform.InverseTransformPoint(mFormationPos).x;

        if (distToFormationPosZ > 0 && distToFormationPosZ > maxDrift)
        {
            mSpeedModifier = distToFormationPosZ / maxDrift;
        }
        else if (distToFormationPosZ < 0 && Mathf.Abs(distToFormationPosZ) > maxDrift && mAgent.remainingDistance < maxDrift * 4.0f)
        {
            mSpeedModifier = maxDrift / Mathf.Abs(distToFormationPosZ);
        }
        else
            mSpeedModifier = 1.0f;

        if (distToFormationPosX > maxDrift)
            mSpeedModifier = 1.0f;

        if (mSpeedModifier > maxSpeedModifier)
            mSpeedModifier = maxSpeedModifier;

        mAgent.speed = mOriginalSpeed * mSpeedModifier;
    }

    /// <summary>
    /// Checks where it needs to be in the formation every 4 frames
    /// </summary>
    /// <returns></returns>
    private IEnumerator CheckSquadTargetPosition()
    {
        yield return null;
        mMyLeader.RegisterMember(this);

        while (mMyLeader.enabled)
        {
            yield return null;

            Vector3 nextTarget;
            mFormationPos = mMyLeader.GetUnitPosition(this, out nextTarget);

            //Don't path if next target is less than repathDistance away from previous target
            if ((nextTarget - mTargetPos).sqrMagnitude > (repathDistance * repathDistance))
            {
                mTargetPos = nextTarget;
                mAgent.SetDestination(mTargetPos);
            }
        }
    }
}
